@@ -24,6 +24,11 @@ install("fastdtw")
 install ("spicy")
 install("cdlib")
 install("community")
+install("python-igraph")
+install("leidenalg")
+install("cdlib")
+#install("infomap==1.0.0b54")
+
 ##----------------------------------------------IMPORTS---------------------------------------------------------------------------#	
 import json
 import itertools
@@ -45,7 +50,12 @@ import os
 from datetime import timedelta
 from datetime import datetime
 import community as community_louvain
-#from cdlib[C] import algorithms
+import leidenalg
+import igraph
+from igraph import *
+from cdlib import algorithms
+#from infomap import Infomap
+from networkx.algorithms import community
 
 
 #---------------------------------------------NLP-Part-------------------------------------------------------------------------------#
@@ -95,8 +105,13 @@ def profile_connections (data_tweets):
             else:
                 screen_name_mention.append(["Empty"])
     data["screen_name"] = screen_name    
-    data["screen_name_mention"] = screen_name_mention  
-    #Agruping
+    data["screen_name_mention"] = screen_name_mention
+    # {col:np.repeat(data[col].values, data[lst_col].str.len()) for col in data.columns.difference([lst_col])} --> Las variables diferentes a la objetivo se conservan y se crean varias filas iguales al tamaño de cada
+    # una de las listas que hay en cada fila
+    # data[lst_col].str.len() --> Longitud de cada lista en cada fila
+    # data.columns.difference([lst_col]) -->Columnas diferentes a la evaluada
+    # .assign(**{lst_col:np.concatenate(data[lst_col].values)} --> Concatena todos los screen_name y asigna la nueva columna
+    # [data.columns.tolist()] --> Dejar el mismo orden de variables
     lst_col = "screen_name_mention"
     data = data[["screen_name","screen_name_mention"]]
 	#Create new rows in case of a tweet has different mentions
@@ -122,16 +137,60 @@ def building_node_list(df):
     
     
 #---------------------------------Undirect Graph ------------------------------------------------
+def node_list_conversion(df_graph):
+    nodes = []
+    idd=[]
+    node_list=pd.DataFrame()
+    for i in range(len(df_graph)):
+        nodes.append(df_graph['screen_name'].iloc[i])
+        nodes.append(df_graph['screen_name_mention'].iloc[i])
+    #Create dataframe, drop duplicates and create a new variable called "id"
+    node_list['screen_name']= nodes
+    node_list=node_list.drop_duplicates(keep='first')
+    node_list = node_list[node_list["screen_name"] !="Empty"].reset_index(drop=True)
+
+    for i in range(len(node_list)):
+        idd.append(i)
+    node_list['id']=idd
+
+    #Create dictionary
+    my_dictionary = {node_list["screen_name"][0]: 0}
+    for i in range(len(node_list)):
+        my_dictionary[str(node_list["screen_name"][i])] = node_list["id"][i]
+    my_dictionary["Empty"] = len(node_list)
+    df_graph["screen_name_mention_str"] = df_graph["screen_name_mention"]
+    df_graph["screen_name_str"] = df_graph["screen_name"]
+    for index, row in df_graph.iterrows():
+        df_graph.loc[index, 'screen_name'] = my_dictionary[row["screen_name"]]
+        df_graph.loc[index, 'screen_name_mention'] = my_dictionary[row["screen_name_mention"]]
+    
+    return df_graph, node_list
+    
+
+
 def buildingGraph(df_graph):
     G =nx.Graph()
     for i in range(len(df_graph)):
         G.add_edges_from([(df_graph.screen_name.iloc[i], df_graph.screen_name_mention.iloc[i])])
     return G
 
-def community_algortihm(df_graph):
 
+def girvanNewman_alg(node_list,df_graph):
+    DiG = building_DiGraph(node_list,df_graph)
+    communities_generator = community.girvan_newman(DiG)
+    top_level_communities = next(communities_generator)
+    next_level_communities = next(communities_generator)
+    communities= sorted(map(sorted, next_level_communities))
+    data = pd.DataFrame()
+    data["screen_name"]=  communities
+    data["community"] = list(range(0,len(communities)))    
+    lst_col ="screen_name"
+    df_algComu =pd.DataFrame({col:np.repeat(data[col].values, data[lst_col].str.len()) for col in data.columns.difference(
+            [lst_col])}).assign(**{lst_col:np.concatenate(data[lst_col].values)})[data.columns.tolist()]
+    return df_algComu
+    
+def louvain_alg(node_list,df_graph):
     G = buildingGraph(df_graph)
-    # compute the best partition
     partition = community_louvain.best_partition(G)
     l=[]
     for (key,value) in partition.items():
@@ -139,15 +198,78 @@ def community_algortihm(df_graph):
 
     df_algComu=pd.DataFrame(data=l,columns= ['screen_name','community'])
     return df_algComu
+    
+    
+
+def leiden_alg (df_graph):
+    df_graph, node_list = node_list_conversion(df_graph)
+    DiG = building_DiGraph(list(node_list["id"]),df_graph)
+    coms = algorithms.leiden(DiG)
+    data = pd.DataFrame()
+    data["id"]=  coms.communities
+    data["community"] = list(range(0,len(coms.communities)))
+    lst_col ="id"
+    df_algComu =pd.DataFrame({col:np.repeat(data[col].values, data[lst_col].str.len()) for col in data.columns.difference(
+            [lst_col])}).assign(**{lst_col:np.concatenate(data[lst_col].values)})[data.columns.tolist()]
+    df_algComu = pd.merge(node_list, df_algComu, on=["id"])
+    return df_algComu
+    
+    
+def infMap_alg(df_graph):
+    # Command line flags can be added as a string to Infomap
+    im = Infomap("--two-level --directed")
+    df_graph, node_list = structure_leiden(df_graph)
+    for i in range(len(df_graph)):
+        im.add_link(df_graph['screen_name'].iloc[i],df_graph['screen_name_mention'].iloc[i])
+    # Run the Infomap search algorithm to find optimal modules
+    im.run()
+    
+    nodes=[]
+    community=[]
+    for node in im.tree:
+        if node.is_leaf:
+            nodes.append(node.node_id)
+            community.append(node.module_id)
+    df_algComu=pd.DataFrame()
+    df_algComu['id']=nodes
+    df_algComu['community']=community
+    df_algComu = pd.merge(node_list, df_algComu, on=["id"])  
+    
+    return df_algComu
+
+
+
+def graph_algoritm_selected(df_graph,selection, node_list):
+
+    len_nodeList = len(node_list)
+    
+    if(selection ==1):
+        df_algComu = girvanNewman_alg(node_list,df_graph)
+    elif(selection ==2):
+        df_algComu = leiden_alg(df_graph)
+    elif(selection ==3):
+        df_algComu = infMap_alg(df_graph)
+    else:#default
+        if(len_nodeList < 1000):
+            df_algComu = girvanNewman_alg(node_list,df_graph)
+        else:
+            df_algComu = leiden_alg(df_graph)
+    
+    return df_algComu
+        
+
+
 
 #------------------------------Directed Graph ---------------------------------------------------
 def building_DiGraph(node_list, df):
     DiG = nx.DiGraph()
     DiG.add_nodes_from(node_list)
     for i in range(len(df)):
-        if(df['screen_name_mention'].iloc[i] !="Empty"):
+        #Second condition for delete empty when we convert screen_name to numbers for specific algorithms
+        if((df['screen_name_mention'].iloc[i] !="Empty") | (df["screen_name_mention"].iloc[i] != len(node_list))):
             DiG.add_edge(df['screen_name'].iloc[i],df['screen_name_mention'].iloc[i])
     return DiG
+    
 
 
 def self_loop_iteration_nodes(df):
